from sqlalchemy import (
    Boolean,
    Column,
    Integer,
    String,
    DateTime,
    Text,
    ForeignKey,
    Enum,
)
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from app.db.base import Base
import enum


class MinistryRole(enum.Enum):
    LEADER = "leader"
    CO_LEADER = "co_leader"
    MEMBER = "member"
    VOLUNTEER = "volunteer"


class Ministry(Base):
    __tablename__ = "ministries"

    id = Column(Integer, primary_key=True, index=True)
    name = Column(String, nullable=False)
    description = Column(Text, nullable=True)
    vision = Column(Text, nullable=True)
    mission = Column(Text, nullable=True)
    meeting_day = Column(String, nullable=True)
    meeting_time = Column(String, nullable=True)
    meeting_location = Column(String, nullable=True)
    leader_id = Column(Integer, ForeignKey("users.id"), nullable=False)
    is_active = Column(Boolean, default=True)
    created_at = Column(DateTime, nullable=False, default=func.now())
    updated_at = Column(
        DateTime, nullable=False, default=func.now(), onupdate=func.now()
    )

    # Relationships
    leader = relationship("User")
    members = relationship(
        "MinistryMember", back_populates="ministry", cascade="all, delete-orphan"
    )
    activities = relationship(
        "MinistryActivity", back_populates="ministry", cascade="all, delete-orphan"
    )


class MinistryMember(Base):
    __tablename__ = "ministry_members"

    id = Column(Integer, primary_key=True, index=True)
    ministry_id = Column(Integer, ForeignKey("ministries.id"), nullable=False)
    user_id = Column(Integer, ForeignKey("users.id"), nullable=False)
    role = Column(Enum(MinistryRole), default=MinistryRole.MEMBER)
    joined_at = Column(DateTime, nullable=False, default=func.now())
    is_active = Column(Boolean, default=True)

    # Relationships
    ministry = relationship("Ministry", back_populates="members")
    user = relationship("User")


class MinistryActivity(Base):
    __tablename__ = "ministry_activities"

    id = Column(Integer, primary_key=True, index=True)
    ministry_id = Column(Integer, ForeignKey("ministries.id"), nullable=False)
    title = Column(String, nullable=False)
    description = Column(Text, nullable=True)
    activity_date = Column(DateTime, nullable=False)
    location = Column(String, nullable=True)
    created_by = Column(Integer, ForeignKey("users.id"), nullable=False)
    created_at = Column(DateTime, nullable=False, default=func.now())

    # Relationships
    ministry = relationship("Ministry", back_populates="activities")
    creator = relationship("User")


class MinistryRequest(Base):
    __tablename__ = "ministry_requests"

    id = Column(Integer, primary_key=True, index=True)
    ministry_id = Column(Integer, ForeignKey("ministries.id"), nullable=False)
    user_id = Column(Integer, ForeignKey("users.id"), nullable=False)
    message = Column(Text, nullable=True)
    status = Column(String, default="pending")  # pending, approved, rejected
    requested_at = Column(DateTime, nullable=False, default=func.now())
    responded_at = Column(DateTime, nullable=True)
    responded_by = Column(Integer, ForeignKey("users.id"), nullable=True)

    # Relationships
    ministry = relationship("Ministry")
    user = relationship("User", foreign_keys=[user_id])
    responder = relationship("User", foreign_keys=[responded_by])
